//---------------------------------------------------------------------------
// Projekt    : Digital Uhr
// Datei      : main.cpp
// Datum      : 05.07.2007
// Version    : 1.0
// Autor      : Karsten Donat
// MCU Typ    : atmega168
//---------------------------------------------------------------------------

#include <avr/io.h>
#include <avr\wdt.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "config.h"
#include "LCD_Text.h"
#include "Standard.h"
#include "uart.h"
#include "Uhr.h"

// RS232 Debug
#define RS232_DEBUG

void init (void)
{
   // Info-LED einschalten whrend Init
   SetBit  (InfoLED_DDR ,InfoLED_PIN); // Info-LED
   ClearBit(InfoLED_PORT,InfoLED_PIN); // LED an (Pin?=0, LED an 5V)

   Uhr_Create(false);


   /*
    *  Initialize UART library, pass baudrate and AVR cpu clock
    *  with the macro 
    *  UART_BAUD_SELECT() (normal speed mode )
    *  or 
    *  UART_BAUD_SELECT_DOUBLE_SPEED() ( double speed mode)
    */
   uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU) ); 


   LCD_Text_Create();

   // PORTD.3 Eingang: DCF Funkempfnger
   ClearBit(DCF77_DDR,DCF77_PIN);
   //SetBit(DCF77_DDR,DCF77_PIN);
   //DCF77_DDR &= ~DCF77_PIN;
   #ifdef RS232_DEBUG
//      print("PORTD.3 Eingang: DCF Funkempfnger\r\n");
   #endif
   delay_ms_grob(200);

   #ifdef RS232_DEBUG
//      print("Initialisierung abgeschlossen.\r\n");
   #endif
   delay_ms_grob(200);

   /*
    * now enable interrupt, since UART library is interrupt controlled
    */
   sei();

    uart_puts("String stored in SRAM\n");
    uart_puts_P("String stored in FLASH\n");


   // Info-LED ausschalten
   SetBit(InfoLED_PORT,InfoLED_PIN); // LED aus (Pin?=1, LED an 5V)

   // Watchdog Timer auf 8s aktivieren
   wdt_enable(WDTO_8S);
}

void Update_Anzeige()
{
   char Buffer[5];

   if (Update_Datum==true)
   {
      Update_Datum=false;
      LCD_GotoXY(22,1);

      ItoA_Zeit( Tag, Buffer);
      LCD_Write_String(Buffer);
      LCD_Write_String(".");
      ItoA_Zeit( Monat, Buffer);
      LCD_Write_String(Buffer);
      LCD_Write_String(".");
      IntToStr( Jahr, Buffer);
      LCD_Write_String(Buffer);
   }

   if (Update_Zeit==true)
   {
      Update_Zeit=false;
      // InfoLED blinken im Sekundenrythmus
      InfoLED_PORT ^= (1<<InfoLED_PIN); //toggle PB0 each time this happens

      //Zeit anzeigen
      LCD_GotoXY(33,1);
//         LCD_Write_String("01.01.2007 ");//00:00:00

      ItoA_Zeit( Stunde, Buffer);
      LCD_Write_String(Buffer);
      LCD_Write_String(":");
      ItoA_Zeit( Minute, Buffer);
      LCD_Write_String(Buffer);
      LCD_Write_String(":");
      ItoA_Zeit( Sekunde, Buffer);
      LCD_Write_String(Buffer);
   } //if
}

int main (void)
{
   // ToDo: Initialisierungsroutinen aufrufen

   init();

   unsigned int c;

   // Variablen zum Uhr stellen per RS232
   Boolean uart_Datum_Zeit=false;
   Byte    uart_Datum_Zeit_Position=1;




   // Select IDLE sleep Mode
   MCUCR &= ~((1<<SM1)|(1<<SM0)|(1<<SM2));


   while (true) // Mainloop
   {
      //ToDo: Hauptprogramm
      wdt_reset(); // WatchDog Timer zurcksetzen


      // Microcontroller schlafen legen //
      // Enable sleep mode
      MCUCR |= (1<<SE);
      // Call "sleep" assembler instruction
      __asm__ __volatile__ ("sleep\n\t"::);
      // Disable sleep mode
      MCUCR &= ~(1<<SE);
      // Microcontroller schlafen legen //



      // UART Interrupt bearbeiten


        /*
         * Get received character from ringbuffer
         * uart_getc() returns in the lower byte the received character and 
         * in the higher byte (bitmask) the last receive error
         * UART_NO_DATA is returned when no data is available.
         *
         */
        c = uart_getc();
        if ( c & UART_NO_DATA )
        {
            /* 
             * no data available from UART 
             */
        }
        else
        {
            /*
             * new data available from UART
             * check for Frame or Overrun error
             */
            if ( c & UART_FRAME_ERROR )
            {
                /* Framing Error detected, i.e no stop bit detected */
                uart_puts_P("UART Frame Error: ");
            }
            if ( c & UART_OVERRUN_ERROR )
            {
                /* 
                 * Overrun, a character already present in the UART UDR register was 
                 * not read by the interrupt handler before the next character arrived,
                 * one or more received characters have been dropped
                 */
                uart_puts_P("UART Overrun Error: ");
            }
            if ( c & UART_BUFFER_OVERFLOW )
            {
                /* 
                 * We are not reading the receive buffer fast enough,
                 * one or more received character have been dropped 
                 */
                uart_puts_P("Buffer overflow error: ");
            }
            /* 
             * send received character back
             */
            uart_putc( (unsigned char)c );

            // Uhrzeit ber RS232 stellen
            // 255 senden, danach jeweils als 1 Byte...  TagMonatJahrStundeMinuteSekunden
            // das Jahr kann von 0 = 2000 bis 254 = 2254 gesendet werden
            if (c==255)
            {
               uart_Datum_Zeit=true;
               uart_Datum_Zeit_Position=1;
            }
            else
            if (uart_Datum_Zeit==true)
            {
               if (uart_Datum_Zeit_Position==1)
               {
                  Tag=(unsigned char)c;
               }
               else
               if (uart_Datum_Zeit_Position==2)
               {
                  Monat=(unsigned char)c;
               }
               else
               if (uart_Datum_Zeit_Position==3)
               {
                  Jahr=2000+((unsigned char)c);
               }
               else
               if (uart_Datum_Zeit_Position==4)
               {
                  Stunde=(unsigned char)c;
               }
               else
               if (uart_Datum_Zeit_Position==5)
               {
                  Minute=(unsigned char)c;
               }
               else
               if (uart_Datum_Zeit_Position==6)
               {
                  Sekunde=(unsigned char)c;
                  uart_Datum_Zeit=false;
                  Update_Datum=true;
               }
               else
               {
                  uart_Datum_Zeit=false;
               }

               uart_Datum_Zeit_Position++;
            }
        }

      Update_Anzeige();
   }

   return 0;
}

